var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _SqsFifoPartialProcessor_instances, _SqsFifoPartialProcessor_currentGroupId, _SqsFifoPartialProcessor_failedGroupIds, _SqsFifoPartialProcessor_addToFailedGroup, _SqsFifoPartialProcessor_processFailRecord, _SqsFifoPartialProcessor_setCurrentGroup;
import { BatchProcessorSync } from './BatchProcessorSync.js';
import { EventType } from './constants.js';
import { SqsFifoMessageGroupShortCircuitError, SqsFifoShortCircuitError, } from './errors.js';
/**
 * Batch processor for SQS FIFO queues
 *
 * This class extends the {@link BatchProcessorSync} class and provides
 * a mechanism to process records from SQS FIFO queues synchronously.
 *
 * By default, we will stop processing at the first failure and mark unprocessed messages as failed to preserve ordering.
 *
 * However, this behavior may not be optimal for customers who wish to proceed with processing messages from a different group ID.
 *
 * @example
 * ```typescript
 * import {
 *   BatchProcessor,
 *   EventType,
 *   processPartialResponseSync,
 * } from '@aws-lambda-powertools/batch';
 * import type { SQSRecord, SQSHandler } from 'aws-lambda';
 *
 * const processor = new BatchProcessor(EventType.SQS);
 *
 * const recordHandler = async (record: SQSRecord): Promise<void> => {
 *   const payload = JSON.parse(record.body);
 * };
 *
 * export const handler: SQSHandler = async (event, context) =>
 *   processPartialResponseSync(event, recordHandler, processor, {
 *     context,
 *   });
 * ```
 */
class SqsFifoPartialProcessor extends BatchProcessorSync {
    constructor() {
        super(EventType.SQS);
        _SqsFifoPartialProcessor_instances.add(this);
        /**
         * The ID of the current message group being processed.
         */
        _SqsFifoPartialProcessor_currentGroupId.set(this, void 0);
        /**
         * A set of group IDs that have already encountered failures.
         */
        _SqsFifoPartialProcessor_failedGroupIds.set(this, void 0);
        __classPrivateFieldSet(this, _SqsFifoPartialProcessor_failedGroupIds, new Set(), "f");
    }
    /**
     * Handles a failure for a given record.
     * Adds the current group ID to the set of failed group IDs if `skipGroupOnError` is true.
     * @param record - The record that failed.
     * @param exception - The error that occurred.
     * @returns The failure response.
     */
    failureHandler(record, exception) {
        if (this.options?.skipGroupOnError && __classPrivateFieldGet(this, _SqsFifoPartialProcessor_currentGroupId, "f")) {
            __classPrivateFieldGet(this, _SqsFifoPartialProcessor_instances, "m", _SqsFifoPartialProcessor_addToFailedGroup).call(this, __classPrivateFieldGet(this, _SqsFifoPartialProcessor_currentGroupId, "f"));
        }
        return super.failureHandler(record, exception);
    }
    /**
     * Process a record with a synchronous handler
     *
     * This method orchestrates the processing of a batch of records synchronously
     * for SQS FIFO queues.
     *
     * The method calls the prepare hook to initialize the processor and then
     * iterates over each record in the batch, processing them one by one.
     *
     * If one of them fails and `skipGroupOnError` is not true, the method short circuits
     * the processing and fails the remaining records in the batch.
     *
     * If one of them fails and `skipGroupOnError` is true, then the method fails the current record
     * if the message group has any previous failure, otherwise keeps processing.
     *
     * Then, it calls the clean hook to clean up the processor and returns the
     * processed records.
     */
    processSync() {
        this.prepare();
        const processedRecords = [];
        let currentIndex = 0;
        for (const record of this.records) {
            __classPrivateFieldGet(this, _SqsFifoPartialProcessor_instances, "m", _SqsFifoPartialProcessor_setCurrentGroup).call(this, record.attributes?.MessageGroupId);
            // If we have any failed messages, we should then short circuit the process and
            // fail remaining messages unless `skipGroupOnError` is true
            const shouldShortCircuit = !this.options?.skipGroupOnError && this.failureMessages.length !== 0;
            if (shouldShortCircuit) {
                return this.shortCircuitProcessing(currentIndex, processedRecords);
            }
            // If `skipGroupOnError` is true and the current group has previously failed,
            // then we should skip processing the current group.
            const shouldSkipCurrentGroup = this.options?.skipGroupOnError &&
                __classPrivateFieldGet(this, _SqsFifoPartialProcessor_currentGroupId, "f") &&
                __classPrivateFieldGet(this, _SqsFifoPartialProcessor_failedGroupIds, "f").has(__classPrivateFieldGet(this, _SqsFifoPartialProcessor_currentGroupId, "f"));
            const result = shouldSkipCurrentGroup
                ? __classPrivateFieldGet(this, _SqsFifoPartialProcessor_instances, "m", _SqsFifoPartialProcessor_processFailRecord).call(this, record, new SqsFifoMessageGroupShortCircuitError())
                : this.processRecordSync(record);
            processedRecords.push(result);
            currentIndex++;
        }
        this.clean();
        return processedRecords;
    }
    /**
     * Starting from the first failure index, fail all remaining messages regardless
     * of their group ID.
     *
     * This short circuit mechanism is used when we detect a failed message in the batch.
     *
     * Since messages in a FIFO queue are processed in order, we must stop processing any
     * remaining messages in the batch to prevent out-of-order processing.
     *
     * @param firstFailureIndex Index of first message that failed
     * @param processedRecords Array of response items that have been processed both successfully and unsuccessfully
     */
    shortCircuitProcessing(firstFailureIndex, processedRecords) {
        const remainingRecords = this.records.slice(firstFailureIndex);
        for (const record of remainingRecords) {
            __classPrivateFieldGet(this, _SqsFifoPartialProcessor_instances, "m", _SqsFifoPartialProcessor_processFailRecord).call(this, record, new SqsFifoShortCircuitError());
        }
        this.clean();
        return processedRecords;
    }
}
_SqsFifoPartialProcessor_currentGroupId = new WeakMap(), _SqsFifoPartialProcessor_failedGroupIds = new WeakMap(), _SqsFifoPartialProcessor_instances = new WeakSet(), _SqsFifoPartialProcessor_addToFailedGroup = function _SqsFifoPartialProcessor_addToFailedGroup(group) {
    __classPrivateFieldGet(this, _SqsFifoPartialProcessor_failedGroupIds, "f").add(group);
}, _SqsFifoPartialProcessor_processFailRecord = function _SqsFifoPartialProcessor_processFailRecord(record, exception) {
    const data = this.toBatchType(record, this.eventType);
    return this.failureHandler(data, exception);
}, _SqsFifoPartialProcessor_setCurrentGroup = function _SqsFifoPartialProcessor_setCurrentGroup(group) {
    __classPrivateFieldSet(this, _SqsFifoPartialProcessor_currentGroupId, group, "f");
};
export { SqsFifoPartialProcessor };
